using MessagePack;
using Newtonsoft.Json;
using ServiceCenter;
using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.Redis;
using ServiceCenter.SqlSugars;
using SqlSugar;
using System.Text;
using WCS.Core;
using WCS.WorkEngineering;

namespace WCS.Service
{
    /// <summary>
    /// 
    /// </summary>
    public class Worker : BackgroundService
    {
        /// <summary>
        /// ¼
        /// </summary>
        private readonly ILogger<Worker> _logger;

        /// <summary>
        /// 캯
        /// </summary>
        /// <param name="logger">¼</param>
        public Worker(ILogger<Worker> logger)
        {
            _logger = logger;
        }

        public static readonly string WcsDlog = "WCSDlog";
        public static readonly string Wcsdb = "WCSDB";

        /// <summary>
        ///  ִ
        /// </summary>
        /// <param name="stoppingToken">ֹͣ</param>
        /// <returns></returns>
        protected override async Task ExecuteAsync(CancellationToken stoppingToken)
        {
            if (stoppingToken.IsCancellationRequested)
                return;

            #region ʼRedis

            var redisConnectionStrings = RedisHub.Default.Check("RedisConnectionStrings") ?? throw new Exception("RedisRedisConnectionStrings");
            var configs = JsonConvert.DeserializeObject<List<DataBaseConnectionString>>(redisConnectionStrings);
            if (configs != null)
            {
                if (configs.All(v => v.Key != "Monitor")) throw new Exception("RedisConnectionStringsüRedisDBַ");
            }

            foreach (var redisConnection in configs!)
            {
                RedisHub.CreateContext(redisConnection.ConnectionString, redisConnection.Key);
                switch (redisConnection.Key)
                {
                    case "Monitor":
                        RedisHub.SetMonitorContextType(redisConnection.Key);
                        RedisHub.Monitor.Serialize = obj =>
                        {
                            var bytes = MessagePackSerializer.Serialize(obj);
                            return bytes;
                        };
                        RedisHub.Monitor.DeserializeRaw = (bytes, type) =>
                        {
                            var obj = MessagePackSerializer.Deserialize(type, bytes);
                            return obj;
                        };
                        break;

                    case "DebugRedisUrl":
                        RedisHub.SetDebugContextType(redisConnection.Key);
                        Configs.DebugRedisUrl = redisConnection.ConnectionString;
                        break;

                    case "WMS":
                        RedisHub.SetWMSContextType(redisConnection.Key);
                        break;
                }
            }

            #endregion ʼRedis

            #region ־

            //var logConfigText = RedisHub.Default.Check("LogConfigText") ?? throw new Exception("Redislog4net");
            //var logConfig = JsonConvert.DeserializeObject<LogConfig>(logConfigText);
            //LogHub.SetConfigInfo(logConfig!);

            #endregion ־

            _logger.LogInformation("WCSʼ");
            Configs.ProtocolProxyBaseType = typeof(ProtocolProxy);
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
            Configs.StringEncoding = Encoding.UTF8;
            //var warehouseName = RedisHub.Default.Check("WarehouseName") ?? throw new Exception("Redisòֿ");
            //if (string.IsNullOrEmpty(warehouseName)) throw new Exception("Redisòֿ");
            //ServiceHub.SetWarehouseName(warehouseName);

            #region ʼݿ

            var dbConnectionStrings = RedisHub.Default.Check("DbConnectionStrings") ?? throw new Exception("Redisݿ");
            ServiceHub.DbConnectionStrings = JsonConvert.DeserializeObject<List<DataBaseConnectionString>>(dbConnectionStrings);
            if (ServiceHub.DbConnectionStrings != null)
            {
                if (ServiceHub.DbConnectionStrings.All(v => v.Key != Wcsdb)) throw new Exception("DbConnectionStringsWCSݿַ");
                if (ServiceHub.DbConnectionStrings.All(v => v.Key == Wcsdb && !v.IsDefault)) throw new Exception("DbConnectionStringsWCSݿΪĬݿ");
                // if (ServiceHub.DbConnectionStrings.All(v => v.Key != WcsDlog)) throw new Exception("DbConnectionStringsWCS־ݿַ");
            }

            //Ϣ
            List<ConnectionConfig> connectionConfigs = new List<ConnectionConfig>();
            foreach (var connectionString in ServiceHub.DbConnectionStrings!)
            {
                connectionConfigs.Add(new ConnectionConfig()
                {
                    ConfigId = connectionString.Key,
                    ConnectionString = connectionString.ConnectionString,//ӷִ
                    DbType = connectionString.DbType,//ݿ
                    IsAutoCloseConnection = true,//trueҪֶclose
                });
            };
            SqlSugarHelper.SetDb(new SqlSugarScope(connectionConfigs));

            ServiceHub.DbConnectionStrings.InitDB();

            #endregion ʼݿ

            #region ʼ豸Ϣ

            WorkStart.InitializeDeviceInfo();

            #endregion ʼ豸Ϣ

            #region ʼPLCPLCȡЭ

            //PLC
            Configs.PLCAccessorCreater = new PLCAccessors.PLCAccessorsCreater();

            try
            {
                #region е

                World.StartAll();

                #endregion е

                _logger.LogInformation("WCSɹ");
            }
            catch (Exception ex)
            {
                _logger.LogError("WCSʧ{0}", ex.Message);
            }

            #endregion ʼPLCPLCȡЭ

            LogHub.init();
        }
    }
}
